Package: optimParallel
Type: Package
Title: Parallel Version of the L-BFGS-B Optimization Method
Version: 1.0-3
Date: 2026-07-29
Authors@R: c(person("Florian", "Gerber", role = c("aut", "cre"),
                    email = "flora.fauna.gerber@gmail.com",
                    comment = c(ORCID = "0000-0001-8545-5263")),
             person("Reinhard", "Furrer", role = "ctb",
                    email = "reinhard.furrer@math.uzh.ch",
                    comment = c(ORCID = "0000-0002-6319-2332")))
Maintainer: Florian Gerber <flora.fauna.gerber@gmail.com>
Description: Provides a parallel version of the L-BFGS-B method of optim(). The main function of the package is optimParallel(), which has the same usage and output as optim(). Using optimParallel() can significantly reduce the optimization time.
License: GPL (>= 2)
URL: https://github.com/florafauna/optimParallel-R
BugReports: https://github.com/florafauna/optimParallel-R/issues
Depends: R (>= 3.5), stats, parallel
Suggests: R.rsp, roxygen2, spam, microbenchmark, testthat, ggplot2,
        numDeriv, lbfgsb3c
VignetteBuilder: R.rsp
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-29 11:48:23 UTC; florz
Author: Florian Gerber [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8545-5263>),
  Reinhard Furrer [ctb] (ORCID: <https://orcid.org/0000-0002-6319-2332>)
Repository: CRAN
Date/Publication: 2026-07-30 05:20:02 UTC
