# Generated by roxygen2: do not edit by hand

export(optimParallel)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,getDefaultCluster)
importFrom(parallel,parLapply)
importFrom(stats,optim)
